# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 13.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V46
37
38
162
163
176
177
1545
1548
1642
1643
2546
2548
2553
2554
3449
3450
8240
8248
8359
8360
8374
8375
8379
8380
8382
8383
8451
8452
8457
8458
43064
43065
65020
65021
65130
65131
65285
65286
65504
65505
73693
73697
126124
126125
126128
126129
END
