+{
   locale_version => 1.27,
   upper_before_lower => 1,
   entry => <<'ENTRY', # for DUCET v10.0.0
010B      ; [.1CDF.0020.0002] # LATIN SMALL LETTER C WITH DOT ABOVE
0063 0307 ; [.1CDF.0020.0002] # LATIN SMALL LETTER C WITH DOT ABOVE
010A      ; [.1CDF.0020.0008] # LATIN CAPITAL LETTER C WITH DOT ABOVE
0043 0307 ; [.1CDF.0020.0008] # LATIN CAPITAL LETTER C WITH DOT ABOVE
0121      ; [.1D59.0020.0002] # LATIN SMALL LETTER G WITH DOT ABOVE
0067 0307 ; [.1D59.0020.0002] # LATIN SMALL LETTER G WITH DOT ABOVE
0120      ; [.1D59.0020.0008] # LATIN CAPITAL LETTER G WITH DOT ABOVE
0047 0307 ; [.1D59.0020.0008] # LATIN CAPITAL LETTER G WITH DOT ABOVE
0067 0127 ; [.1D7D.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER H WITH STROKE>
0067 0126 ; [.1D7D.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER G, LATIN CAPITAL LETTER H WITH STROKE>
0047 0127 ; [.1D7D.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER H WITH STROKE>
0047 0126 ; [.1D7D.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER H WITH STROKE>
0067 0068 0335 ; [.1D7D.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER H, COMBINING SHORT STROKE OVERLAY>
0067 0048 0335 ; [.1D7D.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER G, LATIN CAPITAL LETTER H, COMBINING SHORT STROKE OVERLAY>
0047 0068 0335 ; [.1D7D.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER H, COMBINING SHORT STROKE OVERLAY>
0047 0048 0335 ; [.1D7D.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER H, COMBINING SHORT STROKE OVERLAY>
0127      ; [.1D97.0020.0002] # LATIN SMALL LETTER H WITH STROKE
0068 0335 ; [.1D97.0020.0002] # LATIN SMALL LETTER H WITH STROKE
210F      ; [.1D97.0020.0002] # PLANCK CONSTANT OVER TWO PI
0126      ; [.1D97.0020.0008] # LATIN CAPITAL LETTER H WITH STROKE
0048 0335 ; [.1D97.0020.0008] # LATIN CAPITAL LETTER H WITH STROKE
017C      ; [.1F86.0020.0002] # LATIN SMALL LETTER Z WITH DOT ABOVE
007A 0307 ; [.1F86.0020.0002] # LATIN SMALL LETTER Z WITH DOT ABOVE
017B      ; [.1F86.0020.0008] # LATIN CAPITAL LETTER Z WITH DOT ABOVE
005A 0307 ; [.1F86.0020.0008] # LATIN CAPITAL LETTER Z WITH DOT ABOVE
ENTRY
};
