# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 13.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V78
97
98
102
103
104
107
108
109
110
111
115
117
119
120
121
122
700
701
702
703
768
770
775
777
778
779
780
781
787
788
817
818
834
835
940
941
942
943
945
946
951
952
953
954
961
962
965
966
969
970
974
975
1381
1382
1387
1388
1389
1390
1396
1397
1398
1399
1406
1407
1410
1411
7936
7944
7968
7976
8032
8040
8048
8049
8052
8053
8060
8061
END
